#include <rootvis.h>
#include <xmms/configfile.h>

void color_triplet2arr(char* res, char* triplet) {
	if (sscanf(triplet, "#%2hhx%2hhx%2hhx", &res[0], &res[1], &res[2]) != 3)
		fprintf(stderr, "Color value %s could not be recognized as #rrggbb, ranging from #000000 to #FFFFFF\n", triplet);
/*	if (sscanf(triplet, "(%hhd,%hhd,%hhd)", &res[0], &res[1], &res[2]) != 3)
		fprintf(stderr, "Color value %s could not be recognized in the value (x,y,z) whereby x,y,z are between 0 and 255\n", triplet); */
}

char* color_arr2triplet(unsigned char* src, char* triplet) {
	sprintf(triplet, "#%2.2hhx%2.2hhx%2.2hhx", src[0], src[1], src[2]);
	return triplet;
}

void config_write(void) {
	ConfigFile *fp;
	gchar *filename;
	char colortmp[7];
	print_status("Writing configuration");
	filename = g_strconcat(getenv("HOME"), "/.xmms/config", NULL);
	fp = xmms_cfg_open_file(filename);
	if (fp) {
		xmms_cfg_write_int(fp, "rootvis", "geometry_posx", conf.geo.posx);
		xmms_cfg_write_int(fp, "rootvis", "geometry_posy", conf.geo.posy);
		xmms_cfg_write_int(fp, "rootvis", "geometry_orientation", conf.geo.orientation);
		xmms_cfg_write_int(fp, "rootvis", "geometry_height", conf.geo.height);
		xmms_cfg_write_int(fp, "rootvis", "geometry_space", conf.geo.space);
		xmms_cfg_write_int(fp, "rootvis", "bar_width", conf.bar.width);
		xmms_cfg_write_int(fp, "rootvis", "bar_shadow", conf.bar.shadow);
		xmms_cfg_write_int(fp, "rootvis", "bar_falloff", conf.bar.falloff);
		xmms_cfg_write_int(fp, "rootvis", "peak_enabled", conf.peak.enabled);
		xmms_cfg_write_int(fp, "rootvis", "peak_falloff", conf.peak.falloff);
		xmms_cfg_write_int(fp, "rootvis", "peak_step", conf.peak.step);
		xmms_cfg_write_int(fp, "rootvis", "cutoff", conf.cutoff);
		xmms_cfg_write_int(fp, "rootvis", "div", conf.div);
		xmms_cfg_write_float(fp, "rootvis", "linearity", conf.linearity);
		xmms_cfg_write_int(fp, "rootvis", "fps", conf.fps);
		xmms_cfg_write_string(fp, "rootvis", "bar_color_1", color_arr2triplet(conf.bar.color[0], colortmp));
		xmms_cfg_write_string(fp, "rootvis", "bar_color_2", color_arr2triplet(conf.bar.color[1], colortmp));
		xmms_cfg_write_string(fp, "rootvis", "bar_color_3", color_arr2triplet(conf.bar.color[2], colortmp));
		xmms_cfg_write_string(fp, "rootvis", "bar_color_4", color_arr2triplet(conf.bar.color[3], colortmp));
		xmms_cfg_write_string(fp, "rootvis", "bar_shadow_color", color_arr2triplet(conf.bar.shadow_color, colortmp));
		xmms_cfg_write_string(fp, "rootvis", "peak_color", color_arr2triplet(conf.peak.color, colortmp));
		xmms_cfg_write_int(fp, "rootvis", "debug", conf.debug);
		xmms_cfg_write_file(fp, filename);
		xmms_cfg_free(fp);
	} else	fprintf(stderr, "Could not open xmms configuration file: %s\n", filename);
	g_free(filename);
}

// this parses ~/.xmms/config
// if a setting is not found, it is created to make it possible to edit the default settings
// after the configuration dialogue is finished, this won't be necessary any more
void config_read(void) {
	int dirty = 0;
	ConfigFile *fp;
	gchar *filename, *colortmp;

	conf.geo.posx = DEFAULT_geometry_posx;
	conf.geo.posy = DEFAULT_geometry_posy;
	conf.geo.orientation = DEFAULT_geometry_orientation;
	conf.geo.height = DEFAULT_geometry_height;
	conf.geo.space = DEFAULT_geometry_space;
	conf.bar.width = DEFAULT_bar_width;
	conf.bar.shadow = DEFAULT_bar_shadow;
	conf.bar.falloff = DEFAULT_bar_falloff;
	color_triplet2arr(conf.bar.color[0], DEFAULT_bar_color_1);
	color_triplet2arr(conf.bar.color[1], DEFAULT_bar_color_2);
	color_triplet2arr(conf.bar.color[2], DEFAULT_bar_color_3);
	color_triplet2arr(conf.bar.color[3], DEFAULT_bar_color_4);
	color_triplet2arr(conf.bar.shadow_color, DEFAULT_bar_shadow_color);
	conf.peak.enabled = DEFAULT_peak_enabled;
	conf.peak.falloff = DEFAULT_peak_falloff;
	conf.peak.step = DEFAULT_peak_step;
	color_triplet2arr(conf.peak.color, DEFAULT_peak_color);
	conf.cutoff = DEFAULT_cutoff;
	conf.div = DEFAULT_div;
	conf.linearity = DEFAULT_linearity;
	conf.fps = DEFAULT_fps;
	conf.debug = DEFAULT_debug;
	
	print_status("Reading configuration");
	
	/* without gtk it would be
	   filename = strcat(getenv("HOME"), "/.xmms/config");
	   after determining the length */
	filename = g_strconcat(getenv("HOME"), "/.xmms/config", NULL);
	fp = xmms_cfg_open_file(filename);

	if (fp) {

		if (!(xmms_cfg_read_int(fp, "rootvis", "geometry_posx", &conf.geo.posx))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "geometry_posy", &conf.geo.posy))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "geometry_orientation", &conf.geo.orientation))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "geometry_height", &conf.geo.height))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "geometry_space", &conf.geo.space))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "bar_width", &conf.bar.width))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "bar_shadow", &conf.bar.shadow))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "bar_falloff", &conf.bar.falloff))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "peak_enabled", &conf.peak.enabled))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "peak_falloff", &conf.peak.falloff))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "peak_step", &conf.peak.step))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "cutoff", &conf.cutoff))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "div", &conf.div))) dirty++;
		if (!(xmms_cfg_read_float(fp, "rootvis", "linearity", &conf.linearity))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "fps", &conf.fps))) dirty++;
		if (!(xmms_cfg_read_int(fp, "rootvis", "debug", &conf.debug))) dirty++;
		if (!(xmms_cfg_read_string(fp, "rootvis", "bar_color_1", &colortmp))) dirty++;
		 else	color_triplet2arr(conf.bar.color[0], colortmp);
		if (!(xmms_cfg_read_string(fp, "rootvis", "bar_color_2", &colortmp))) dirty++;
		 else	color_triplet2arr(conf.bar.color[1], colortmp);
		if (!(xmms_cfg_read_string(fp, "rootvis", "bar_color_3", &colortmp))) dirty++;
		 else	color_triplet2arr(conf.bar.color[2], colortmp);
		if (!(xmms_cfg_read_string(fp, "rootvis", "bar_color_4", &colortmp))) dirty++;
		 else	color_triplet2arr(conf.bar.color[3], colortmp);
		if (!(xmms_cfg_read_string(fp, "rootvis", "bar_shadow_color", &colortmp))) dirty++;
		 else	color_triplet2arr(conf.bar.shadow_color, colortmp);
		if (!(xmms_cfg_read_string(fp, "rootvis", "peak_color", &colortmp))) dirty++;
		 else	color_triplet2arr(conf.peak.color, colortmp);

		xmms_cfg_free(fp);
		if (dirty > 0) config_write();
	} else	fprintf(stderr, "Could not open xmms configuration file: %s\n", filename);
	g_free(filename);
	print_status("Configuration finished");
}
